@@ -1,19 +1,12 @@
-/*-------------------------------------------------------------------------
+/** Implementation of libpqxx STL-style cursor classes.
  *
- *   FILE
- *	cursor.cxx
+ * These classes wrap SQL cursors in STL-like interfaces.
  *
- *   DESCRIPTION
- *      implementation of libpqxx STL-style cursor classes.
- *   These classes wrap SQL cursors in STL-like interfaces
- *
- * Copyright (c) 2004-2013, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2004-2017, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
  * or contact the author.
- *
- *-------------------------------------------------------------------------
  */
 #include "pqxx/compiler-internal.hxx"
 
@@ -30,7 +23,6 @@
 #include "pqxx/internal/gates/icursorstream-icursor_iterator.hxx"
 #include "pqxx/internal/gates/result-sql_cursor.hxx"
 
-using namespace PGSTD;
 using namespace pqxx;
 using namespace pqxx::internal;
 
@@ -45,27 +37,41 @@ inline bool useless_trail(char c)
 {
   return isspace(c) || c==';';
 }
-
 }
 
 
-pqxx::internal::sql_cursor::sql_cursor(transaction_base &t,
-	const string &query,
-	const string &cname,
+pqxx::cursor_base::difference_type pqxx::cursor_base::all() noexcept
+{
+  // Implemented out-of-line so we don't fall afoul of Visual Studio defining
+  // min() and max() macros, which turn this expression into malformed code:
+  return std::numeric_limits<int>::max() - 1;
+}
+
+
+pqxx::cursor_base::difference_type cursor_base::backward_all() noexcept
+{
+  // Implemented out-of-line so we don't fall afoul of Visual Studio defining
+  // min() and max() macros, which turn this expression into malformed code:
+  return std::numeric_limits<int>::min() + 1;
+}
+
+
+pqxx::internal::sql_cursor::sql_cursor(
+	transaction_base &t,
+	const std::string &query,
+	const std::string &cname,
 	cursor_base::accesspolicy ap,
 	cursor_base::updatepolicy up,
 	cursor_base::ownershippolicy op,
 	bool hold) :
   cursor_base(t.conn(), cname),
   m_home(t.conn()),
-  m_empty_result(),
   m_adopted(false),
   m_at_end(-1),
-  m_pos(0),
-  m_endpos(-1)
+  m_pos(0)
 {
   if (&t.conn() != &m_home) throw internal_error("Cursor in wrong connection");
-  stringstream cq, qn;
+  std::stringstream cq, qn;
 
   /* Strip trailing semicolons (and whitespace, as side effect) off query.  The
    * whitespace is stripped because it might otherwise mask a semicolon.  After
@@ -73,7 +79,7 @@ pqxx::internal::sql_cursor::sql_cursor(transaction_base &t,
    * query.begin() and last, i.e. last may be equal to query.end() or point to
    * the first useless trailing character.
    */
-  string::const_iterator last = query.end();
+  auto last = query.end();
   // TODO: May break on multibyte encodings!
   for (--last; last!=query.begin() && useless_trail(*last); --last) ;
   if (last==query.begin() && useless_trail(*last))
@@ -83,30 +89,17 @@ pqxx::internal::sql_cursor::sql_cursor(transaction_base &t,
   cq << "DECLARE \"" << name() << "\" ";
 
   m_home.activate();
-  if (m_home.supports(connection_base::cap_cursor_scroll))
-  {
-    if (ap == cursor_base::forward_only) cq << "NO ";
-    cq << "SCROLL ";
-  }
+
+  if (ap == cursor_base::forward_only) cq << "NO ";
+  cq << "SCROLL ";
 
   cq << "CURSOR ";
 
-  if (hold)
-  {
-    if (!m_home.supports(connection_base::cap_cursor_with_hold))
-      throw failure("Cursor " + name() + " "
-	  "created for use outside of its originating transaction, "
-	  "but this backend version does not support that.");
-    cq << "WITH HOLD ";
-  }
+  if (hold) cq << "WITH HOLD ";
 
-  cq << "FOR " << string(query.begin(),last) << ' ';
+  cq << "FOR " << std::string(query.begin(),last) << ' ';
 
   if (up != cursor_base::update) cq << "FOR READ ONLY ";
-  else if (!m_home.supports(connection_base::cap_cursor_update))
-    throw failure("Cursor " + name() + " "
-	"created as updatable, "
-	"but this backend version does not support that.");
   else cq << "FOR UPDATE ";
 
   qn << "[DECLARE " << name() << ']';
@@ -128,19 +121,19 @@ pqxx::internal::sql_cursor::sql_cursor(transaction_base &t,
 }
 
 
-pqxx::internal::sql_cursor::sql_cursor(transaction_base &t,
-	const string &cname,
+pqxx::internal::sql_cursor::sql_cursor(
+	transaction_base &t,
+	const std::string &cname,
 	cursor_base::ownershippolicy op) :
   cursor_base(t.conn(), cname, false),
   m_home(t.conn()),
   m_empty_result(),
   m_adopted(true),
   m_at_end(0),
-  m_pos(-1),
-  m_endpos(-1)
+  m_pos(-1)
 {
-  // If we take responsibility for destroying the cursor, that's one less reason
-  // not to allow the connection to be deactivated and reactivated.
+  // If we take responsibility for destroying the cursor, that's one less
+  // reason not to allow the connection to be deactivated and reactivated.
   // TODO: Go over lifetime/reactivation rules again to be sure they work
   if (op==cursor_base::owned)
     gate::connection_sql_cursor(t.conn()).add_reactivation_avoidance_count(-1);
@@ -149,17 +142,17 @@ pqxx::internal::sql_cursor::sql_cursor(transaction_base &t,
 }
 
 
-void pqxx::internal::sql_cursor::close() throw ()
+void pqxx::internal::sql_cursor::close() noexcept
 {
   if (m_ownership==cursor_base::owned)
   {
     try
     {
-      gate::connection_sql_cursor(m_home).Exec(
+      gate::connection_sql_cursor(m_home).exec(
 	("CLOSE \"" + name() + "\"").c_str(),
 	0);
     }
-    catch (const exception &)
+    catch (const std::exception &)
     {
     }
 
@@ -174,10 +167,7 @@ void pqxx::internal::sql_cursor::close() throw ()
 void pqxx::internal::sql_cursor::init_empty_result(transaction_base &t)
 {
   if (pos() != 0) throw internal_error("init_empty_result() from bad pos()");
-
-  // This doesn't work with older backends, where "FETCH 0" meant "FETCH ALL."
-  if (m_home.supports(connection_base::cap_cursor_fetch_0))
-    m_empty_result = t.exec("FETCH 0 IN \"" + name() + '"');
+  m_empty_result = t.exec("FETCH 0 IN \"" + name() + '"');
 }
 
 
@@ -209,7 +199,8 @@ pqxx::internal::sql_cursor::adjust(difference_type hoped,
     if (direction > 0) hit_end = true;
     else if (m_pos == -1) m_pos = actual;
     else if (m_pos != actual)
-      throw internal_error("Moved back to beginning, but wrong position: "
+      throw internal_error(
+	"Moved back to beginning, but wrong position: "
         "hoped=" + to_string(hoped) + ", "
         "actual=" + to_string(actual) + ", "
         "m_pos=" + to_string(m_pos) + ", "
@@ -233,7 +224,8 @@ pqxx::internal::sql_cursor::adjust(difference_type hoped,
 }
 
 
-result pqxx::internal::sql_cursor::fetch(difference_type rows,
+result pqxx::internal::sql_cursor::fetch(
+	difference_type rows,
 	difference_type &displacement)
 {
   if (!rows)
@@ -241,8 +233,9 @@ result pqxx::internal::sql_cursor::fetch(difference_type rows,
     displacement = 0;
     return m_empty_result;
   }
-  const string query = "FETCH " + stridestring(rows) + " IN \"" + name() + "\"";
-  const result r(gate::connection_sql_cursor(m_home).Exec(query.c_str(), 0));
+  const std::string query =
+      "FETCH " + stridestring(rows) + " IN \"" + name() + "\"";
+  const result r(gate::connection_sql_cursor(m_home).exec(query.c_str(), 0));
   displacement = adjust(rows, difference_type(r.size()));
   return r;
 }
@@ -258,51 +251,34 @@ cursor_base::difference_type pqxx::internal::sql_cursor::move(
     return 0;
   }
 
-  const string query = "MOVE " + stridestring(rows) + " IN \"" + name() + "\"";
-  const result r(gate::connection_sql_cursor(m_home).Exec(query.c_str(), 0));
-
-  // Starting with the libpq in PostgreSQL 7.4, PQcmdTuples() (which we call
-  // indirectly here) also returns the number of rows skipped by a MOVE
-  difference_type d = static_cast<difference_type>(r.affected_rows());
-
-  // We may not have PQcmdTuples(), or this may be a libpq version that doesn't
-  // implement it for MOVE yet.  We'll also get zero if we decide to use a
-  // prepared statement for the MOVE.
-  if (!d)
-  {
-    static const string StdResponse("MOVE ");
-    const char *const status = gate::result_sql_cursor(r).CmdStatus();
-    if (strncmp(status, StdResponse.c_str(), StdResponse.size()) != 0)
-      throw internal_error("cursor MOVE returned "
-	  "'" + string(status) + "' "
-	  "(expected '" + StdResponse + "')");
-
-    from_string(status + StdResponse.size(), d);
-  }
-
+  const std::string query =
+      "MOVE " + stridestring(rows) + " IN \"" + name() + "\"";
+  const result r(gate::connection_sql_cursor(m_home).exec(query.c_str(), 0));
+  difference_type d = difference_type(r.affected_rows());
   displacement = adjust(rows, d);
   return d;
 }
 
 
-string pqxx::internal::sql_cursor::stridestring(difference_type n)
+std::string pqxx::internal::sql_cursor::stridestring(difference_type n)
 {
   /* Some special-casing for ALL and BACKWARD ALL here.  We used to use numeric
    * "infinities" for difference_type for this (the highest and lowest possible
    * values for "long"), but for PostgreSQL 8.0 at least, the backend appears to
    * expect a 32-bit number and fails to parse large 64-bit numbers.
-   * We could change the typedef to match this behaviour, but that would break
+   * We could change the alias to match this behaviour, but that would break
    * if/when Postgres is changed to accept 64-bit displacements.
    */
-  static const string All("ALL"), BackAll("BACKWARD ALL");
+  static const std::string All("ALL"), BackAll("BACKWARD ALL");
   if (n >= cursor_base::all()) return All;
   else if (n <= cursor_base::backward_all()) return BackAll;
   return to_string(n);
 }
 
 
-pqxx::cursor_base::cursor_base(connection_base &context,
-	const PGSTD::string &Name,
+pqxx::cursor_base::cursor_base(
+	connection_base &context,
+	const std::string &Name,
 	bool embellish_name) :
   m_name(embellish_name ? context.adorn_name(Name) : Name)
 {
@@ -338,8 +314,8 @@ result pqxx::internal::stateless_cursor_retrieve(
 
 pqxx::icursorstream::icursorstream(
     transaction_base &context,
-    const PGSTD::string &query,
-    const PGSTD::string &basename,
+    const std::string &query,
+    const std::string &basename,
     difference_type sstride) :
   m_cur(context,
 	query,
@@ -351,7 +327,7 @@ pqxx::icursorstream::icursorstream(
   m_stride(sstride),
   m_realpos(0),
   m_reqpos(0),
-  m_iterators(0),
+  m_iterators(nullptr),
   m_done(false)
 {
   set_stride(sstride);
@@ -360,14 +336,14 @@ pqxx::icursorstream::icursorstream(
 
 pqxx::icursorstream::icursorstream(
     transaction_base &context,
-    const result::field &cname,
+    const field &cname,
     difference_type sstride,
     cursor_base::ownershippolicy op) :
   m_cur(context, cname.c_str(), op),
   m_stride(sstride),
   m_realpos(0),
   m_reqpos(0),
-  m_iterators(0),
+  m_iterators(nullptr),
   m_done(false)
 {
   set_stride(sstride);
@@ -390,9 +366,9 @@ result pqxx::icursorstream::fetchblock()
 }
 
 
-icursorstream &pqxx::icursorstream::ignore(PGSTD::streamsize n)
+icursorstream &pqxx::icursorstream::ignore(std::streamsize n)
 {
-  difference_type offset = m_cur.move(n);
+  auto offset = m_cur.move(difference_type(n));
   m_realpos += offset;
   if (offset < n) m_done = true;
   return *this;
@@ -402,11 +378,11 @@ icursorstream &pqxx::icursorstream::ignore(PGSTD::streamsize n)
 icursorstream::size_type pqxx::icursorstream::forward(size_type n)
 {
   m_reqpos += difference_type(n) * m_stride;
-  return size_type(m_reqpos);
+  return icursorstream::size_type(m_reqpos);
 }
 
 
-void pqxx::icursorstream::insert_iterator(icursor_iterator *i) throw ()
+void pqxx::icursorstream::insert_iterator(icursor_iterator *i) noexcept
 {
   gate::icursor_iterator_icursorstream(*i).set_next(m_iterators);
   if (m_iterators)
@@ -415,23 +391,23 @@ void pqxx::icursorstream::insert_iterator(icursor_iterator *i) throw ()
 }
 
 
-void pqxx::icursorstream::remove_iterator(icursor_iterator *i) const throw ()
+void pqxx::icursorstream::remove_iterator(icursor_iterator *i) const noexcept
 {
   gate::icursor_iterator_icursorstream igate(*i);
   if (i == m_iterators)
   {
     m_iterators = igate.get_next();
     if (m_iterators)
-      gate::icursor_iterator_icursorstream(*m_iterators).set_prev(0);
+      gate::icursor_iterator_icursorstream(*m_iterators).set_prev(nullptr);
   }
   else
   {
-    icursor_iterator *prev = igate.get_prev(), *next = igate.get_next();
+    auto prev = igate.get_prev(), next = igate.get_next();
     gate::icursor_iterator_icursorstream(*prev).set_next(next);
     if (next) gate::icursor_iterator_icursorstream(*next).set_prev(prev);
   }
-  igate.set_prev(0);
-  igate.set_next(0);
+  igate.set_prev(nullptr);
+  igate.set_next(nullptr);
 }
 
 
@@ -439,20 +415,20 @@ void pqxx::icursorstream::service_iterators(difference_type topos)
 {
   if (topos < m_realpos) return;
 
-  typedef multimap<difference_type,icursor_iterator*> todolist;
+  using todolist = std::multimap<difference_type,icursor_iterator*>;
   todolist todo;
   for (icursor_iterator *i = m_iterators, *next; i; i = next)
   {
     gate::icursor_iterator_icursorstream gate(*i);
-    const icursor_iterator::difference_type ipos = gate.pos();
+    const auto ipos = gate.pos();
     if (ipos >= m_realpos && ipos <= topos)
       todo.insert(todolist::value_type(ipos, i));
     next = gate.get_next();
   }
-  const todolist::const_iterator todo_end(todo.end());
-  for (todolist::const_iterator i = todo.begin(); i != todo_end; )
+  const auto todo_end = todo.end();
+  for (auto i = todo.begin(); i != todo_end; )
   {
-    const difference_type readpos = i->first;
+    const auto readpos = i->first;
     if (readpos > m_realpos) ignore(readpos - m_realpos);
     const result r = fetchblock();
     for ( ; i != todo_end && i->first == readpos; ++i)
@@ -461,38 +437,32 @@ void pqxx::icursorstream::service_iterators(difference_type topos)
 }
 
 
-pqxx::icursor_iterator::icursor_iterator() throw () :
-  m_stream(0),
-  m_here(),
-  m_pos(0),
-  m_prev(0),
-  m_next(0)
+pqxx::icursor_iterator::icursor_iterator() noexcept :
+  m_pos(0)
 {
 }
 
-pqxx::icursor_iterator::icursor_iterator(istream_type &s) throw () :
+
+pqxx::icursor_iterator::icursor_iterator(istream_type &s) noexcept :
   m_stream(&s),
-  m_here(),
-  m_pos(difference_type(gate::icursorstream_icursor_iterator(s).forward(0))),
-  m_prev(0),
-  m_next(0)
+  m_pos(difference_type(gate::icursorstream_icursor_iterator(s).forward(0)))
 {
   gate::icursorstream_icursor_iterator(*m_stream).insert_iterator(this);
 }
 
-pqxx::icursor_iterator::icursor_iterator(const icursor_iterator &rhs) throw () :
+
+pqxx::icursor_iterator::icursor_iterator(const icursor_iterator &rhs)
+	noexcept :
   m_stream(rhs.m_stream),
   m_here(rhs.m_here),
-  m_pos(rhs.m_pos),
-  m_prev(0),
-  m_next(0)
+  m_pos(rhs.m_pos)
 {
-  if (m_stream) 
+  if (m_stream)
     gate::icursorstream_icursor_iterator(*m_stream).insert_iterator(this);
 }
 
 
-pqxx::icursor_iterator::~icursor_iterator() throw ()
+pqxx::icursor_iterator::~icursor_iterator() noexcept
 {
   if (m_stream)
     gate::icursorstream_icursor_iterator(*m_stream).remove_iterator(this);
@@ -526,14 +496,15 @@ icursor_iterator &pqxx::icursor_iterator::operator+=(difference_type n)
     throw argument_error("Advancing icursor_iterator by negative offset");
   }
   m_pos = difference_type(
-	gate::icursorstream_icursor_iterator(*m_stream).forward(size_type(n)));
+	gate::icursorstream_icursor_iterator(*m_stream).forward(
+		icursorstream::size_type(n)));
   m_here.clear();
   return *this;
 }
 
 
 icursor_iterator &
-pqxx::icursor_iterator::operator=(const icursor_iterator &rhs) throw ()
+pqxx::icursor_iterator::operator=(const icursor_iterator &rhs) noexcept
 {
   if (rhs.m_stream == m_stream)
   {
