@@ -1,22 +1,17 @@
-/*-------------------------------------------------------------------------
+/** Implementation of the pqxx::pipeline class.
  *
- *   FILE
- *	pipeline.cxx
+ * Throughput-optimized query interface.
  *
- *   DESCRIPTION
- *      implementation of the pqxx::pipeline class
- *   Throughput-optimized query manager
- *
- * Copyright (c) 2003-2013, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2003-2017, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
  * or contact the author.
- *
- *-------------------------------------------------------------------------
  */
 #include "pqxx/compiler-internal.hxx"
 
+#include <iterator>
+
 #include "pqxx/dbtransaction"
 #include "pqxx/pipeline"
 
@@ -24,7 +19,6 @@
 #include "pqxx/internal/gates/result-creation.hxx"
 
 
-using namespace PGSTD;
 using namespace pqxx;
 using namespace pqxx::internal;
 
@@ -32,34 +26,24 @@ using namespace pqxx::internal;
 
 namespace
 {
-const string theSeparator("; ");
-const string theDummyValue("1");
-const string theDummyQuery("SELECT " + theDummyValue + theSeparator);
+const std::string theSeparator("; ");
+const std::string theDummyValue("1");
+const std::string theDummyQuery("SELECT " + theDummyValue + theSeparator);
 }
 
 
-pqxx::pipeline::pipeline(transaction_base &t, const PGSTD::string &Name) :
+pqxx::pipeline::pipeline(transaction_base &t, const std::string &Name) :
   namedclass("pipeline", Name),
-  transactionfocus(t),
-  m_queries(),
-  m_issuedrange(),
-  m_retain(0),
-  m_num_waiting(0),
-  m_q_id(0),
-  m_dummy_pending(false),
-  m_error(qid_limit())
+  transactionfocus(t)
 {
   m_issuedrange = make_pair(m_queries.end(), m_queries.end());
   attach();
 }
 
 
-pqxx::pipeline::~pipeline() throw ()
+pqxx::pipeline::~pipeline() noexcept
 {
-#ifdef PQXX_QUIET_DESTRUCTORS
-  disable_noticer Quiet(m_Trans.conn());
-#endif
-  try { cancel(); } catch (const exception &) {}
+  try { cancel(); } catch (const std::exception &) {}
   detach();
 }
 
@@ -76,13 +60,13 @@ void pqxx::pipeline::detach()
 }
 
 
-pipeline::query_id pqxx::pipeline::insert(const PGSTD::string &q)
+pipeline::query_id pqxx::pipeline::insert(const std::string &q)
 {
   attach();
   const query_id qid = generate_id();
   pqxxassert(qid > 0);
   pqxxassert(m_queries.lower_bound(qid)==m_queries.end());
-  const QueryMap::iterator i = m_queries.insert(make_pair(qid,Query(q))).first;
+  const auto i = m_queries.insert(std::make_pair(qid,Query(q))).first;
 
   if (m_issuedrange.second == m_queries.end())
   {
@@ -141,8 +125,8 @@ void pqxx::pipeline::cancel()
 {
   while (have_pending())
   {
-    gate::connection_pipeline(m_Trans.conn()).cancel_query();
-    QueryMap::iterator canceled_query = m_issuedrange.first;
+    gate::connection_pipeline(m_trans.conn()).cancel_query();
+    auto canceled_query = m_issuedrange.first;
     ++m_issuedrange.first;
     m_queries.erase(canceled_query);
   }
@@ -152,16 +136,18 @@ void pqxx::pipeline::cancel()
 bool pqxx::pipeline::is_finished(pipeline::query_id q) const
 {
   if (m_queries.find(q) == m_queries.end())
-    throw logic_error("Requested status for unknown query " + to_string(q));
-  return (QueryMap::const_iterator(m_issuedrange.first)==m_queries.end()) ||
-         (q < m_issuedrange.first->first && q < m_error);
+    throw std::logic_error(
+      "Requested status for unknown query " + to_string(q));
+  return
+	(QueryMap::const_iterator(m_issuedrange.first)==m_queries.end()) ||
+	(q < m_issuedrange.first->first && q < m_error);
 }
 
 
-pair<pipeline::query_id, result> pqxx::pipeline::retrieve()
+std::pair<pipeline::query_id, result> pqxx::pipeline::retrieve()
 {
   if (m_queries.empty())
-    throw logic_error("Attempt to retrieve result from empty pipeline");
+    throw std::logic_error("Attempt to retrieve result from empty pipeline");
   return retrieve(m_queries.begin());
 }
 
@@ -169,7 +155,8 @@ pair<pipeline::query_id, result> pqxx::pipeline::retrieve()
 int pqxx::pipeline::retain(int retain_max)
 {
   if (retain_max < 0)
-    throw range_error("Attempt to make pipeline retain " +
+    throw range_error(
+	"Attempt to make pipeline retain " +
 	to_string(retain_max) + " queries");
 
   const int oldvalue = m_retain;
@@ -195,7 +182,7 @@ void pqxx::pipeline::resume()
 pipeline::query_id pqxx::pipeline::generate_id()
 {
   if (m_q_id == qid_limit())
-    throw overflow_error("Too many queries went through pipeline");
+    throw std::overflow_error("Too many queries went through pipeline");
   ++m_q_id;
   return m_q_id;
 }
@@ -211,24 +198,25 @@ void pqxx::pipeline::issue()
 
   // TODO: Wrap in nested transaction if available, for extra "replayability"
 
-  // Retrieve that NULL result for the last query, if needed
+  // Retrieve that null result for the last query, if needed
   obtain_result();
 
   // Don't issue anything if we've encountered an error
   if (m_error < qid_limit()) return;
 
   // Start with oldest query (lowest id) not in previous issue range
-  QueryMap::iterator oldest = m_issuedrange.second;
+  auto oldest = m_issuedrange.second;
   pqxxassert(oldest != m_queries.end());
 
   // Construct cumulative query string for entire batch
-  string cum = separated_list(theSeparator,oldest,m_queries.end(),getquery());
-  const QueryMap::size_type num_issued = QueryMap::size_type(
-    internal::distance(oldest, m_queries.end()));
+  std::string cum = separated_list(
+          theSeparator, oldest, m_queries.end(), getquery());
+  const auto num_issued = QueryMap::size_type(std::distance(
+	oldest, m_queries.end()));
   const bool prepend_dummy = (num_issued > 1);
   if (prepend_dummy) cum = theDummyQuery + cum;
 
-  gate::connection_pipeline(m_Trans.conn()).start_exec(cum);
+  gate::connection_pipeline(m_trans.conn()).start_exec(cum);
 
   // Since we managed to send out these queries, update state to reflect this
   m_dummy_pending = prepend_dummy;
@@ -238,8 +226,7 @@ void pqxx::pipeline::issue()
 }
 
 
-void pqxx::pipeline::internal_error(const PGSTD::string &err)
-	throw (logic_error)
+void pqxx::pipeline::internal_error(const std::string &err)
 {
   set_error_at(0);
   throw pqxx::internal_error(err);
@@ -251,8 +238,8 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
   pqxxassert(!m_dummy_pending);
   pqxxassert(!m_queries.empty());
 
-  gate::connection_pipeline gate(m_Trans.conn());
-  internal::pq::PGresult *r = gate.get_result();
+  gate::connection_pipeline gate(m_trans.conn());
+  const auto r = gate.get_result();
   if (!r)
   {
     if (have_pending() && !expect_none)
@@ -265,15 +252,13 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
 
   pqxxassert(r);
   const result res = gate::result_creation::create(
-	r,
-	0,
-	m_queries.begin()->second.get_query(),
-	gate::connection_pipeline(m_Trans.conn()).encoding_code());
+	r, m_queries.begin()->second.get_query());
 
   if (!have_pending())
   {
     set_error_at(m_queries.begin()->first);
-    throw logic_error("Got more results from pipeline than there were queries");
+    throw std::logic_error(
+      "Got more results from pipeline than there were queries");
   }
 
   // Must be the result for the oldest pending query
@@ -290,23 +275,19 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
 void pqxx::pipeline::obtain_dummy()
 {
   pqxxassert(m_dummy_pending);
-  gate::connection_pipeline gate(m_Trans.conn());
-  internal::pq::PGresult *const r = gate.get_result();
+  gate::connection_pipeline gate(m_trans.conn());
+  const auto r = gate.get_result();
   m_dummy_pending = false;
 
   if (!r)
     internal_error("pipeline got no result from backend when it expected one");
 
-  result R = gate::result_creation::create(
-	r,
-	0,
-	"[DUMMY PIPELINE QUERY]",
-	gate::connection_pipeline(m_Trans.conn()).encoding_code());
+  result R = gate::result_creation::create(r, "[DUMMY PIPELINE QUERY]");
 
   bool OK = false;
   try
   {
-    gate::result_creation(R).CheckStatus();
+    gate::result_creation(R).check_status();
     OK = true;
   }
   catch (const sql_error &)
@@ -317,7 +298,7 @@ void pqxx::pipeline::obtain_dummy()
     if (R.size() > 1)
       internal_error("unexpected result for dummy query in pipeline");
 
-    if (string(R.at(0).at(0).c_str()) != theDummyValue)
+    if (std::string(R.at(0).at(0).c_str()) != theDummyValue)
       internal_error("dummy query in pipeline returned unexpected value");
     return;
   }
@@ -329,19 +310,18 @@ void pqxx::pipeline::obtain_dummy()
    */
   // First, give the whole batch the same syntax error message, in case all else
   // is going to fail.
-  for (QueryMap::iterator i = m_issuedrange.first;
-       i != m_issuedrange.second;
-       ++i) i->second.set_result(R);
+  for (auto i = m_issuedrange.first; i != m_issuedrange.second; ++i)
+    i->second.set_result(R);
 
   // Remember where the end of this batch was
-  const QueryMap::iterator stop = m_issuedrange.second;
+  const auto stop = m_issuedrange.second;
 
-  // Retrieve that NULL result for the last query, if needed
+  // Retrieve that null result for the last query, if needed
   obtain_result(true);
 
 
   // Reset internal state to forget botched batch attempt
-  m_num_waiting += int(internal::distance(m_issuedrange.first, stop));
+  m_num_waiting += int(std::distance(m_issuedrange.first, stop));
   m_issuedrange.second = m_issuedrange.first;
 
   pqxxassert(!m_dummy_pending);
@@ -355,26 +335,26 @@ void pqxx::pipeline::obtain_dummy()
     do
     {
       m_num_waiting--;
-      const string &query = m_issuedrange.first->second.get_query();
-      const result res(m_Trans.exec(query));
+      const std::string &query = m_issuedrange.first->second.get_query();
+      const result res(m_trans.exec(query));
       m_issuedrange.first->second.set_result(res);
-      gate::result_creation(res).CheckStatus();
+      gate::result_creation(res).check_status();
       ++m_issuedrange.first;
     }
     while (m_issuedrange.first != stop);
   }
-  catch (const exception &)
+  catch (const std::exception &)
   {
     pqxxassert(m_issuedrange.first != m_queries.end());
 
     const query_id thud = m_issuedrange.first->first;
     ++m_issuedrange.first;
     m_issuedrange.second = m_issuedrange.first;
-    QueryMap::const_iterator q = m_issuedrange.first;
+    auto q = m_issuedrange.first;
     set_error_at( (q == m_queries.end()) ?  thud + 1 : q->first);
 
-    pqxxassert(m_num_waiting ==
-      internal::distance(m_issuedrange.second, m_queries.end()));
+    pqxxassert(
+	m_num_waiting == std::distance(m_issuedrange.second, m_queries.end()));
   }
 
   pqxxassert(m_issuedrange.first != m_queries.end());
@@ -382,21 +362,21 @@ void pqxx::pipeline::obtain_dummy()
 }
 
 
-PGSTD::pair<pipeline::query_id, result>
+std::pair<pipeline::query_id, result>
 pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
 {
   if (q == m_queries.end())
-    throw logic_error("Attempt to retrieve result for unknown query");
+    throw std::logic_error("Attempt to retrieve result for unknown query");
 
   if (q->first >= m_error)
-    throw runtime_error("Could not complete query in pipeline "
-	"due to error in earlier query");
+    throw std::runtime_error(
+	"Could not complete query in pipeline due to error in earlier query");
 
   // If query hasn't issued yet, do it now
   if (m_issuedrange.second != m_queries.end() &&
       (q->first >= m_issuedrange.second->first))
   {
-    pqxxassert(internal::distance(m_issuedrange.second, q) >= 0);
+    pqxxassert(std::distance(m_issuedrange.second, q) >= 0);
 
     if (have_pending()) receive(m_issuedrange.second);
     if (m_error == qid_limit()) issue();
@@ -407,7 +387,7 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
   {
     if (q->first >= m_issuedrange.first->first)
     {
-      QueryMap::const_iterator suc = q;
+      auto suc = q;
       ++suc;
       receive(suc);
     }
@@ -420,18 +400,18 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
   pqxxassert((m_error <= q->first) || (q != m_issuedrange.first));
 
   if (q->first >= m_error)
-    throw runtime_error("Could not complete query in pipeline "
-	"due to error in earlier query");
+    throw std::runtime_error(
+	"Could not complete query in pipeline due to error in earlier query");
 
   // Don't leave the backend idle if there are queries waiting to be issued
   if (m_num_waiting && !have_pending() && (m_error==qid_limit())) issue();
 
   const result R = q->second.get_result();
-  pair<query_id,result> P(make_pair(q->first, R));
+  const auto P = std::make_pair(q->first, R);
 
   m_queries.erase(q);
 
-  gate::result_creation(R).CheckStatus();
+  gate::result_creation(R).check_status();
   return P;
 }
 
@@ -439,7 +419,7 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
 void pqxx::pipeline::get_further_available_results()
 {
   pqxxassert(!m_dummy_pending);
-  gate::connection_pipeline gate(m_Trans.conn());
+  gate::connection_pipeline gate(m_trans.conn());
   while (!gate.is_busy() && obtain_result())
     if (!gate.consume_input()) throw broken_connection();
 }
@@ -447,7 +427,7 @@ void pqxx::pipeline::get_further_available_results()
 
 void pqxx::pipeline::receive_if_available()
 {
-  gate::connection_pipeline gate(m_Trans.conn());
+  gate::connection_pipeline gate(m_trans.conn());
   if (!gate.consume_input()) throw broken_connection();
   if (gate.is_busy()) return;
 
@@ -469,5 +449,3 @@ void pqxx::pipeline::receive(pipeline::QueryMap::const_iterator stop)
   if (QueryMap::const_iterator(m_issuedrange.first) == stop)
     get_further_available_results();
 }
-
-
