@@ -1,30 +1,19 @@
-/*-------------------------------------------------------------------------
+/** Common code and definitions for the transaction classes.
  *
- *   FILE
- *	transaction_base.cxx
+ * pqxx::transaction_base defines the interface for any abstract class that
+ * represents a database transaction.
  *
- *   DESCRIPTION
- *      common code and definitions for the transaction classes
- *   pqxx::transaction_base defines the interface for any abstract class that
- *   represents a database transaction
- *
- * Copyright (c) 2001-2011, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2017, Jeroen T. Vermeulen.
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
  * or contact the author.
- *
- *-------------------------------------------------------------------------
  */
 #include "pqxx/compiler-internal.hxx"
 
 #include <cstring>
 #include <stdexcept>
 
-#ifdef PQXX_QUIET_DESTRUCTORS
-#include "pqxx/errorhandler"
-#endif
-
 #include "pqxx/connection_base"
 #include "pqxx/result"
 #include "pqxx/transaction_base"
@@ -34,13 +23,12 @@
 #include "pqxx/internal/gates/transaction-transactionfocus.hxx"
 
 
-using namespace PGSTD;
 using namespace pqxx::internal;
 
 
 pqxx::internal::parameterized_invocation::parameterized_invocation(
 	connection_base &c,
-	const PGSTD::string &query) :
+	const std::string &query) :
   m_home(c),
   m_query(query)
 {
@@ -49,63 +37,55 @@ pqxx::internal::parameterized_invocation::parameterized_invocation(
 
 pqxx::result pqxx::internal::parameterized_invocation::exec()
 {
-  scoped_array<const char *> values;
-  scoped_array<int> lengths;
-  scoped_array<int> binaries;
+  std::vector<const char *> values;
+  std::vector<int> lengths;
+  std::vector<int> binaries;
   const int elements = marshall(values, lengths, binaries);
 
   return gate::connection_parameterized_invocation(m_home).parameterized_exec(
 	m_query,
-	values.get(),
-	lengths.get(),
-	binaries.get(),
+	&values[0],
+	&lengths[0],
+	&binaries[0],
 	elements);
 }
 
 
 pqxx::transaction_base::transaction_base(connection_base &C, bool direct) :
   namedclass("transaction_base"),
-  m_reactivation_avoidance(),
-  m_Conn(C),
-  m_Focus(),
-  m_Status(st_nascent),
-  m_Registered(false),
-  m_PendingError()
+  m_conn(C)
 {
   if (direct)
   {
     gate::connection_transaction gate(conn());
-    gate.RegisterTransaction(this);
-    m_Registered = true;
+    gate.register_transaction(this);
+    m_registered = true;
   }
 }
 
 
 pqxx::transaction_base::~transaction_base()
 {
-#ifdef PQXX_QUIET_DESTRUCTORS
-  quiet_errorhandler quiet(m_Conn);
-#endif
   try
   {
     reactivation_avoidance_clear();
-    if (!m_PendingError.empty())
-      process_notice("UNPROCESSED ERROR: " + m_PendingError + "\n");
+    if (!m_pending_error.empty())
+      process_notice("UNPROCESSED ERROR: " + m_pending_error + "\n");
 
-    if (m_Registered)
+    if (m_registered)
     {
-      m_Conn.process_notice(description() + " was never closed properly!\n");
+      m_conn.process_notice(description() + " was never closed properly!\n");
       gate::connection_transaction gate(conn());
-      gate.UnregisterTransaction(this);
+      gate.unregister_transaction(this);
     }
   }
-  catch (const exception &e)
+  catch (const std::exception &e)
   {
     try
     {
-      process_notice(string(e.what()) + "\n");
+      process_notice(std::string(e.what()) + "\n");
     }
-    catch (const exception &)
+    catch (const std::exception &)
     {
       process_notice(e.what());
     }
@@ -119,7 +99,7 @@ void pqxx::transaction_base::commit()
 
   // Check previous status code.  Caller should only call this function if
   // we're in "implicit" state, but multiple commits are silently accepted.
-  switch (m_Status)
+  switch (m_status)
   {
   case st_nascent:	// Empty transaction.  No skin off our nose.
     return;
@@ -136,14 +116,14 @@ void pqxx::transaction_base::commit()
     // that an abort is needed--which would only confuse things further at this
     // stage.
     // Therefore, multiple commits are accepted, though under protest.
-    m_Conn.process_notice(description() + " committed more than once\n");
+    m_conn.process_notice(description() + " committed more than once\n");
     return;
 
   case st_in_doubt:
     // Transaction may or may not have been committed.  The only thing we can
     // really do is keep telling the caller that the transaction is in doubt.
-    throw in_doubt_error(description() +
-		      " committed again while in an indeterminate state");
+    throw in_doubt_error(
+	description() + " committed again while in an indeterminate state");
 
   default:
     throw internal_error("pqxx::transaction: invalid status code");
@@ -153,37 +133,37 @@ void pqxx::transaction_base::commit()
   // the commit() will come before the stream is closed.  Which means the
   // commit is premature.  Punish this swiftly and without fail to discourage
   // the habit from forming.
-  if (m_Focus.get())
-    throw failure("Attempt to commit " + description() + " "
-		"with " + m_Focus.get()->description() + " "
-		"still open");
+  if (m_focus.get())
+    throw failure(
+	"Attempt to commit " + description() + " with " +
+	m_focus.get()->description() + " still open");
 
   // Check that we're still connected (as far as we know--this is not an
   // absolute thing!) before trying to commit.  If the connection was broken
   // already, the commit would fail anyway but this way at least we don't remain
   // in-doubt as to whether the backend got the commit order at all.
-  if (!m_Conn.is_open())
-    throw broken_connection("Broken connection to backend; "
-	"cannot complete transaction");
+  if (!m_conn.is_open())
+    throw broken_connection(
+	"Broken connection to backend; cannot complete transaction");
 
   try
   {
     do_commit();
-    m_Status = st_committed;
+    m_status = st_committed;
   }
   catch (const in_doubt_error &)
   {
-    m_Status = st_in_doubt;
+    m_status = st_in_doubt;
     throw;
   }
-  catch (const exception &)
+  catch (const std::exception &)
   {
-    m_Status = st_aborted;
+    m_status = st_aborted;
     throw;
   }
 
   gate::connection_transaction gate(conn());
-  gate.AddVariables(m_Vars);
+  gate.add_variables(m_vars);
 
   End();
 }
@@ -193,13 +173,13 @@ void pqxx::transaction_base::abort()
 {
   // Check previous status code.  Quietly accept multiple aborts to
   // simplify emergency bailout code.
-  switch (m_Status)
+  switch (m_status)
   {
   case st_nascent:	// Never began transaction.  No need to issue rollback.
     break;
 
   case st_active:
-    try { do_abort(); } catch (const exception &) { }
+    try { do_abort(); } catch (const std::exception &) { }
     break;
 
   case st_aborted:
@@ -211,28 +191,28 @@ void pqxx::transaction_base::abort()
   case st_in_doubt:
     // Aborting an in-doubt transaction is probably a reasonably sane response
     // to an insane situation.  Log it, but do not complain.
-    m_Conn.process_notice("Warning: " + description() + " "
-		          "aborted after going into indeterminate state; "
-			  "it may have been executed anyway.\n");
+    m_conn.process_notice(
+	"Warning: " + description() + " aborted after going into "
+	"indeterminate state; it may have been executed anyway.\n");
     return;
 
   default:
     throw internal_error("invalid transaction status");
   }
 
-  m_Status = st_aborted;
+  m_status = st_aborted;
   End();
 }
 
 
-string pqxx::transaction_base::esc_raw(const PGSTD::string &str) const
+std::string pqxx::transaction_base::esc_raw(const std::string &str) const
 {
   const unsigned char *p = reinterpret_cast<const unsigned char *>(str.c_str());
   return conn().esc_raw(p, str.size());
 }
 
 
-string pqxx::transaction_base::quote_raw(const PGSTD::string &str) const
+std::string pqxx::transaction_base::quote_raw(const std::string &str) const
 {
   const unsigned char *p = reinterpret_cast<const unsigned char *>(str.c_str());
   return conn().quote_raw(p, str.size());
@@ -241,7 +221,7 @@ string pqxx::transaction_base::quote_raw(const PGSTD::string &str) const
 
 void pqxx::transaction_base::activate()
 {
-  switch (m_Status)
+  switch (m_status)
   {
   case st_nascent:
     // Make sure transaction has begun before executing anything
@@ -264,18 +244,19 @@ void pqxx::transaction_base::activate()
 }
 
 
-pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
-					  const PGSTD::string &Desc)
+pqxx::result pqxx::transaction_base::exec(
+	const std::string &Query,
+	const std::string &Desc)
 {
   CheckPendingError();
 
-  const string N = (Desc.empty() ? "" : "'" + Desc + "' ");
+  const std::string N = (Desc.empty() ? "" : "'" + Desc + "' ");
 
-  if (m_Focus.get())
-    throw usage_error("Attempt to execute query " + N +
-		      "on " + description() + " "
-		      "with " + m_Focus.get()->description() + " "
-		      "still open");
+  if (m_focus.get())
+    throw usage_error(
+	"Attempt to execute query " + N +
+	"on " + description() + " "
+	"with " + m_focus.get()->description() + " still open");
 
   try
   {
@@ -291,15 +272,60 @@ pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
 }
 
 
+pqxx::result pqxx::transaction_base::exec_n(
+	size_t rows,
+	const std::string &Query,
+	const std::string &Desc)
+{
+  const result r = exec(Query, Desc);
+  if (r.size() != rows)
+  {
+    const std::string N = (Desc.empty() ? "" : "'" + Desc + "'");
+    throw unexpected_rows(
+	"Expected " + to_string(rows) + " row(s) of data "
+        "from query " + N + ", got " + to_string(r.size()) + ".");
+  }
+  return r;
+}
+
+
+void pqxx::transaction_base::check_rowcount_prepared(
+	const std::string &statement,
+	size_t expected_rows,
+	size_t actual_rows)
+{
+  if (actual_rows != expected_rows)
+  {
+    throw unexpected_rows(
+	"Expected " + to_string(expected_rows) + " row(s) of data "
+	"from prepared statement '" + statement + "', got " +
+	to_string(actual_rows) + ".");
+  }
+}
+
+
+void pqxx::transaction_base::check_rowcount_params(
+	size_t expected_rows,
+	size_t actual_rows)
+{
+  if (actual_rows != expected_rows)
+  {
+    throw unexpected_rows(
+	"Expected " + to_string(expected_rows) + " row(s) of data "
+	"from parameterised query, got " + to_string(actual_rows) + ".");
+  }
+}
+
+
 pqxx::internal::parameterized_invocation
-pqxx::transaction_base::parameterized(const PGSTD::string &query)
+pqxx::transaction_base::parameterized(const std::string &query)
 {
   return internal::parameterized_invocation(conn(), query);
 }
 
 
 pqxx::prepare::invocation
-pqxx::transaction_base::prepared(const PGSTD::string &statement)
+pqxx::transaction_base::prepared(const std::string &statement)
 {
   try
   {
@@ -314,39 +340,58 @@ pqxx::transaction_base::prepared(const PGSTD::string &statement)
 }
 
 
-void pqxx::transaction_base::set_variable(const PGSTD::string &Var,
-                                          const PGSTD::string &Value)
+pqxx::result pqxx::transaction_base::internal_exec_prepared(
+	const std::string &statement,
+	const internal::params &args)
 {
-  // Before committing to this new value, see what the backend thinks about it
   gate::connection_transaction gate(conn());
-  gate.RawSetVar(Var, Value);
-  m_Vars[Var] = Value;
+  return gate.exec_prepared(statement, args);
 }
 
 
-string pqxx::transaction_base::get_variable(const PGSTD::string &Var)
+pqxx::result pqxx::transaction_base::internal_exec_params(
+	const std::string &query,
+	const internal::params &args)
 {
-  const map<string,string>::const_iterator i = m_Vars.find(Var);
-  if (i != m_Vars.end()) return i->second;
-  return gate::connection_transaction(conn()).RawGetVar(Var);
+  gate::connection_transaction gate(conn());
+  return gate.exec_params(query, args);
+}
+
+
+void pqxx::transaction_base::set_variable(
+	const std::string &Var,
+	const std::string &Value)
+{
+  // Before committing to this new value, see what the backend thinks about it
+  gate::connection_transaction gate(conn());
+  gate.raw_set_var(Var, Value);
+  m_vars[Var] = Value;
+}
+
+
+std::string pqxx::transaction_base::get_variable(const std::string &Var)
+{
+  const std::map<std::string,std::string>::const_iterator i = m_vars.find(Var);
+  if (i != m_vars.end()) return i->second;
+  return gate::connection_transaction(conn()).raw_get_var(Var);
 }
 
 
 void pqxx::transaction_base::Begin()
 {
-  if (m_Status != st_nascent)
-    throw internal_error("pqxx::transaction: "
-		      "Begin() called while not in nascent state");
+  if (m_status != st_nascent)
+    throw internal_error(
+	"pqxx::transaction: Begin() called while not in nascent state");
 
   try
   {
     // Better handle any pending notifications before we begin
-    m_Conn.get_notifs();
+    m_conn.get_notifs();
 
     do_begin();
-    m_Status = st_active;
+    m_status = st_active;
   }
-  catch (const exception &)
+  catch (const std::exception &)
   {
     End();
     throw;
@@ -354,80 +399,78 @@ void pqxx::transaction_base::Begin()
 }
 
 
-
-void pqxx::transaction_base::End() throw ()
+void pqxx::transaction_base::End() noexcept
 {
   try
   {
     try { CheckPendingError(); }
-    catch (const exception &e) { m_Conn.process_notice(e.what()); }
+    catch (const std::exception &e) { m_conn.process_notice(e.what()); }
 
-    if (m_Registered)
+    if (m_registered)
     {
-      m_Registered = false;
+      m_registered = false;
       gate::connection_transaction gate(conn());
-      gate.UnregisterTransaction(this);
+      gate.unregister_transaction(this);
     }
 
-    if (m_Status != st_active) return;
+    if (m_status != st_active) return;
 
-    if (m_Focus.get())
-      m_Conn.process_notice("Closing " + description() + " "
-			    " with " + m_Focus.get()->description() + " "
-			    "still open\n");
+    if (m_focus.get())
+      m_conn.process_notice(
+	"Closing " + description() + "  with " +
+	m_focus.get()->description() + " still open\n");
 
     try { abort(); }
-    catch (const exception &e) { m_Conn.process_notice(e.what()); }
+    catch (const std::exception &e) { m_conn.process_notice(e.what()); }
 
     gate::connection_transaction gate(conn());
     gate.take_reactivation_avoidance(m_reactivation_avoidance.get());
     m_reactivation_avoidance.clear();
   }
-  catch (const exception &e)
+  catch (const std::exception &e)
   {
-    try { m_Conn.process_notice(e.what()); } catch (const exception &) {}
+    try { m_conn.process_notice(e.what()); } catch (const std::exception &) {}
   }
 }
 
 
-
-void pqxx::transaction_base::RegisterFocus(internal::transactionfocus *S)
+void pqxx::transaction_base::register_focus(internal::transactionfocus *S)
 {
-  m_Focus.Register(S);
+  m_focus.register_guest(S);
 }
 
 
-void pqxx::transaction_base::UnregisterFocus(internal::transactionfocus *S)
-	throw ()
+void pqxx::transaction_base::unregister_focus(internal::transactionfocus *S)
+	noexcept
 {
   try
   {
-    m_Focus.Unregister(S);
+    m_focus.unregister_guest(S);
   }
-  catch (const exception &e)
+  catch (const std::exception &e)
   {
-    m_Conn.process_notice(string(e.what()) + "\n");
+    m_conn.process_notice(std::string(e.what()) + "\n");
   }
 }
 
 
-pqxx::result pqxx::transaction_base::DirectExec(const char C[], int Retries)
+pqxx::result pqxx::transaction_base::direct_exec(const char C[], int Retries)
 {
   CheckPendingError();
-  return gate::connection_transaction(conn()).Exec(C, Retries);
+  return gate::connection_transaction(conn()).exec(C, Retries);
 }
 
 
-void pqxx::transaction_base::RegisterPendingError(const PGSTD::string &Err)
-	throw ()
+void pqxx::transaction_base::register_pending_error(const std::string &Err)
+	noexcept
 {
-  if (m_PendingError.empty() && !Err.empty())
+  if (m_pending_error.empty() && !Err.empty())
   {
     try
     {
-      m_PendingError = Err;
+      m_pending_error = Err;
     }
-    catch (const exception &e)
+    catch (const std::exception &e)
     {
       try
       {
@@ -446,83 +489,83 @@ void pqxx::transaction_base::RegisterPendingError(const PGSTD::string &Err)
 
 void pqxx::transaction_base::CheckPendingError()
 {
-  if (!m_PendingError.empty())
+  if (!m_pending_error.empty())
   {
-    const string Err(m_PendingError);
-#ifdef PQXX_HAVE_STRING_CLEAR
-    m_PendingError.clear();
-#else
-    m_PendingError.resize(0);
-#endif
-    throw failure(m_PendingError);
+    const std::string Err(m_pending_error);
+    m_pending_error.clear();
+    throw failure(Err);
   }
 }
 
 
 namespace
 {
-string MakeCopyString(const string &Table, const string &Columns)
+std::string MakeCopyString(
+        const std::string &Table,
+        const std::string &Columns)
 {
-  string Q = "COPY " + Table + " ";
+  std::string Q = "COPY " + Table + " ";
   if (!Columns.empty()) Q += "(" + Columns + ") ";
   return Q;
 }
 } // namespace
 
 
-void pqxx::transaction_base::BeginCopyRead(const PGSTD::string &Table,
-    const PGSTD::string &Columns)
+void pqxx::transaction_base::BeginCopyRead(
+	const std::string &Table,
+	const std::string &Columns)
 {
   exec(MakeCopyString(Table, Columns) + "TO STDOUT");
 }
 
 
-void pqxx::transaction_base::BeginCopyWrite(const PGSTD::string &Table,
-    const PGSTD::string &Columns)
+void pqxx::transaction_base::BeginCopyWrite(
+	const std::string &Table,
+	const std::string &Columns)
 {
   exec(MakeCopyString(Table, Columns) + "FROM STDIN");
 }
 
 
-bool pqxx::transaction_base::ReadCopyLine(PGSTD::string &line)
+bool pqxx::transaction_base::read_copy_line(std::string &line)
 {
-  return gate::connection_transaction(conn()).ReadCopyLine(line);
+  return gate::connection_transaction(conn()).read_copy_line(line);
 }
 
 
-void pqxx::transaction_base::WriteCopyLine(const PGSTD::string &line)
+void pqxx::transaction_base::write_copy_line(const std::string &line)
 {
   gate::connection_transaction gate(conn());
-  gate.WriteCopyLine(line);
+  gate.write_copy_line(line);
 }
 
 
-void pqxx::transaction_base::EndCopyWrite()
+void pqxx::transaction_base::end_copy_write()
 {
   gate::connection_transaction gate(conn());
-  gate.EndCopyWrite();
+  gate.end_copy_write();
 }
 
 
 void pqxx::internal::transactionfocus::register_me()
 {
-  gate::transaction_transactionfocus gate(m_Trans);
-  gate.RegisterFocus(this);
+  gate::transaction_transactionfocus gate(m_trans);
+  gate.register_focus(this);
   m_registered = true;
 }
 
 
-void pqxx::internal::transactionfocus::unregister_me() throw ()
+void pqxx::internal::transactionfocus::unregister_me() noexcept
 {
-  gate::transaction_transactionfocus gate(m_Trans);
-  gate.UnregisterFocus(this);
+  gate::transaction_transactionfocus gate(m_trans);
+  gate.unregister_focus(this);
   m_registered = false;
 }
 
 void
-pqxx::internal::transactionfocus::reg_pending_error(const PGSTD::string &err)
-	throw ()
+pqxx::internal::transactionfocus::reg_pending_error(const std::string &err)
+	noexcept
 {
-  gate::transaction_transactionfocus gate(m_Trans);
-  gate.RegisterPendingError(err);
+  gate::transaction_transactionfocus gate(m_trans);
+  gate.register_pending_error(err);
 }
