@@ -2,12 +2,12 @@
 
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
 {
-class TestErrorHandler : public errorhandler
+class TestErrorHandler final : public errorhandler
 {
 public:
   TestErrorHandler(
@@ -20,7 +20,7 @@ public:
     handler_list(activated_handlers)
   {}
 
-  virtual bool operator()(const char msg[]) throw ()
+  virtual bool operator()(const char msg[]) noexcept override
   {
     message = string(msg);
     handler_list.push_back(this);
@@ -38,7 +38,6 @@ namespace pqxx
 {
 template<> struct string_traits<TestErrorHandler *>
 {
-  typedef TestErrorHandler *subject_type;
   static const char *name() { return "TestErrorHandler"; }
   static bool has_null() { return true; }
   static bool is_null(TestErrorHandler *t) { return !t; }
@@ -116,17 +115,18 @@ void test_destroying_connection_unregisters_handlers()
 }
 
 
-class MinimalErrorHandler: public errorhandler
+class MinimalErrorHandler final : public errorhandler
 {
 public:
-  MinimalErrorHandler(connection_base &c) : errorhandler(c) {}
-  virtual bool operator()(const char[]) throw () { return true; }
+  explicit MinimalErrorHandler(connection_base &c) : errorhandler(c) {}
+  virtual bool operator()(const char[]) noexcept override
+	{ return true; }
 };
 
 
 void test_get_errorhandlers(connection_base &c)
 {
-  MinimalErrorHandler *eh3 = NULL;
+  MinimalErrorHandler *eh3 = nullptr;
   const vector<errorhandler *> handlers_before = c.get_errorhandlers();
   const size_t base_handlers = handlers_before.size();
 
