@@ -1,6 +1,8 @@
+#include <iostream>
+
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
@@ -38,7 +40,7 @@ void test_esc(connection_base &c, transaction_base &t)
     "x",
     " ",
     "",
-    NULL
+    nullptr
   };
   for (size_t i=0; escstrings[i]; ++i) compare_esc(c, t, escstrings[i]);
 }
@@ -49,7 +51,7 @@ void test_quote(connection_base &c, transaction_base &t)
   PQXX_CHECK_EQUAL(t.quote("x"), "'x'", "Basic quote() fails.");
   PQXX_CHECK_EQUAL(t.quote(1), "'1'", "quote() not dealing with int properly.");
   PQXX_CHECK_EQUAL(t.quote(0), "'0'", "Quoting zero is a problem.");
-  const char *const null_ptr = NULL;
+  const char *const null_ptr = nullptr;
   PQXX_CHECK_EQUAL(t.quote(null_ptr), "NULL", "Not quoting NULL correctly.");
   PQXX_CHECK_EQUAL(t.quote(string("'")), "''''", "Escaping quotes goes wrong.");
 
@@ -67,7 +69,7 @@ void test_quote(connection_base &c, transaction_base &t)
     "\\'",
     "\t",
     "\n",
-    NULL
+    nullptr
   };
 
   for (size_t i=0; test_strings[i]; ++i)
@@ -94,11 +96,35 @@ void test_quote_name(transaction_base &t)
 }
 
 
+void test_esc_raw_unesc_raw(transaction_base &t)
+{
+  const char binary[] = "1\0023\\4x5";
+  const string data(binary, sizeof(binary));
+  const string escaped = t.esc_raw(data);
+
+  cout << "Escaped data: " << escaped << endl;
+
+  for (const auto i: escaped)
+    PQXX_CHECK(isascii(i), "Non-ASCII character in escaped data: " + escaped);
+
+  for (const auto i: escaped)
+    PQXX_CHECK(
+	isprint(i),
+	"Unprintable character in escaped data: " + escaped);
+
+  PQXX_CHECK_EQUAL(
+	t.unesc_raw(escaped),
+	data,
+	"Unescaping binary data does not undo escaping it.");
+}
+
+
 void test_escaping(transaction_base &t)
 {
   test_esc(t.conn(), t);
   test_quote(t.conn(), t);
   test_quote_name(t);
+  test_esc_raw_unesc_raw(t);
 }
 } // namespace
 
