@@ -1,37 +1,43 @@
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
 {
 template<typename T> T make_infinity()
 {
-  return
-#ifdef PQXX_HAVE_LIMITS
-    numeric_limits<T>::infinity();
-#else
-    INFINITY;
-#endif
+  return numeric_limits<T>::infinity();
 }
 
-void infinity_test(transaction_base &)
+/// Test conversions for some floating-point type.
+template<typename T> void infinity_test()
 {
-  double inf = make_infinity<double>();
+  T inf = make_infinity<T>();
   string inf_string;
-  double back_conversion;
+  T back_conversion;
 
   inf_string = to_string(inf);
   from_string(inf_string, back_conversion);
   PQXX_CHECK_LESS(
-	999999999,
+	T(999999999),
 	back_conversion,
 	"Infinity doesn't convert back to something huge.");
 
   inf_string = to_string(-inf);
   from_string(inf_string, back_conversion);
-  PQXX_CHECK_LESS(back_conversion, -999999999, "Negative infinity is broken");
+  PQXX_CHECK_LESS(
+	back_conversion,
+	-T(999999999),
+	"Negative infinity is broken");
+}
+
+void test_infinities(transaction_base &)
+{
+  infinity_test<float>();
+  infinity_test<double>();
+  infinity_test<long double>();
 }
 } // namespace
 
-PQXX_REGISTER_TEST_NODB(infinity_test)
+PQXX_REGISTER_TEST_NODB(test_infinities)
