@@ -1,6 +1,6 @@
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
@@ -12,9 +12,7 @@ void test_read_transaction(transaction_base &trans)
 	1,
 	"Bad result from read transaction.");
 
-  // This error terminates the test on 7.4 backends, so can't test for it there.
-  if (trans.conn().server_version() >= 80000)
-    PQXX_CHECK_THROWS(
+  PQXX_CHECK_THROWS(
 	trans.exec("CREATE TABLE should_not_exist(x integer)"),
 	sql_error,
 	"Read-only transaction allows database to be modified.");
