@@ -1,6 +1,6 @@
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
