@@ -1,6 +1,6 @@
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
@@ -10,7 +10,7 @@ void test_forward_sql_cursor(transaction_base &trans)
   // Plain owned, scoped, forward-only read-only cursor.
   internal::sql_cursor forward(
 	trans,
-	pqxx::test::select_series(trans.conn(), 1, 4),
+	"SELECT generate_series(1, 4)",
 	"forward",
 	cursor_base::forward_only,
 	cursor_base::read_only,
@@ -70,7 +70,7 @@ void test_forward_sql_cursor(transaction_base &trans)
   // Move through entire result set at once.
   internal::sql_cursor forward2(
 	trans,
-	pqxx::test::select_series(trans.conn(), 1, 4),
+	"SELECT generate_series(1, 4)",
 	"forward",
 	cursor_base::forward_only,
 	cursor_base::read_only,
@@ -86,7 +86,7 @@ void test_forward_sql_cursor(transaction_base &trans)
 
   internal::sql_cursor forward3(
 	trans,
-	pqxx::test::select_series(trans.conn(), 1, 4),
+	"SELECT generate_series(1, 4)",
 	"forward",
 	cursor_base::forward_only,
 	cursor_base::read_only,
@@ -102,7 +102,7 @@ void test_forward_sql_cursor(transaction_base &trans)
 
   internal::sql_cursor forward_empty(
 	trans,
-	pqxx::test::select_series(trans.conn(), 0, -1),
+	"SELECT generate_series(0, -1)",
 	"forward_empty",
 	cursor_base::forward_only,
 	cursor_base::read_only,
@@ -120,7 +120,7 @@ void test_scroll_sql_cursor(transaction_base &trans)
 {
   internal::sql_cursor scroll(
 	trans,
-	pqxx::test::select_series(trans.conn(), 1, 10),
+	"SELECT generate_series(1, 10)",
 	"scroll",
 	cursor_base::random_access,
 	cursor_base::read_only,
@@ -179,11 +179,12 @@ void test_scroll_sql_cursor(transaction_base &trans)
   PQXX_CHECK_EQUAL(rows[0][0].as<int>(), 3, "Data position mismatch");
 }
 
-void test_adopted_sql_cursor(connection_base &conn, transaction_base &trans)
+
+void test_adopted_sql_cursor(connection_base &, transaction_base &trans)
 {
   trans.exec(
-	"DECLARE adopted SCROLL CURSOR FOR " +
-	pqxx::test::select_series(conn, 1, 3));
+	"DECLARE adopted SCROLL CURSOR FOR "
+	"SELECT generate_series(1, 3)");
   internal::sql_cursor adopted(trans, "adopted", cursor_base::owned);
   PQXX_CHECK_EQUAL(adopted.pos(), -1, "Adopted cursor has known pos()");
   PQXX_CHECK_EQUAL(adopted.endpos(), -1, "Adopted cursor has known endpos()");
@@ -213,10 +214,9 @@ void test_adopted_sql_cursor(connection_base &conn, transaction_base &trans)
   // Owned adopted cursors are cleaned up on destruction.
   connection conn2;
   work trans2(conn2, "trans2");
-  pqxx::test::prepare_series(trans2, 0, 5);
   trans2.exec(
-	"DECLARE adopted2 CURSOR FOR " +
-	pqxx::test::select_series(conn2, 1, 3));
+	"DECLARE adopted2 CURSOR FOR "
+	"SELECT generate_series(1, 3)");
   {
     internal::sql_cursor(trans2, "adopted2", cursor_base::owned);
   }
@@ -238,10 +238,9 @@ void test_adopted_sql_cursor(connection_base &conn, transaction_base &trans)
   trans2.abort();
 
   work trans3(conn2, "trans3");
-  pqxx::test::prepare_series(trans3, 1, 3);
   trans3.exec(
-	"DECLARE adopted3 CURSOR FOR " +
-	pqxx::test::select_series(conn2, 1, 3));
+	"DECLARE adopted3 CURSOR FOR "
+	"SELECT generate_series(1, 3)");
   {
     internal::sql_cursor(trans3, "adopted3", cursor_base::loose);
   }
@@ -250,13 +249,10 @@ void test_adopted_sql_cursor(connection_base &conn, transaction_base &trans)
 
 void test_hold_cursor(connection_base &conn, transaction_base &trans)
 {
-  // "WITH HOLD" cursors require backend version 7.4 or better.
-  if (conn.server_version() <= 70400) return;
-
   // "With hold" cursor is kept after commit.
   internal::sql_cursor with_hold(
 	trans,
-	pqxx::test::select_series(conn, 1, 3),
+	"SELECT generate_series(1, 3)",
 	"hold_cursor",
 	cursor_base::forward_only,
 	cursor_base::read_only,
@@ -270,7 +266,7 @@ void test_hold_cursor(connection_base &conn, transaction_base &trans)
   // Cursor without hold is closed on commit.
   internal::sql_cursor no_hold(
 	trans2,
-	pqxx::test::select_series(conn, 1, 3),
+	"SELECT generate_series(1, 3)",
 	"no_hold_cursor",
 	cursor_base::forward_only,
 	cursor_base::read_only,
