@@ -1,6 +1,6 @@
 #include <test_helpers.hxx>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
@@ -9,7 +9,7 @@ void test_stateless_cursor(transaction_base &trans)
 {
   stateless_cursor<cursor_base::read_only, cursor_base::owned> empty(
 	trans,
-	pqxx::test::select_series(trans.conn(), 0, -1),
+	"SELECT generate_series(0, -1)",
 	"empty",
 	false);
 
@@ -27,7 +27,7 @@ void test_stateless_cursor(transaction_base &trans)
 
   stateless_cursor<cursor_base::read_only, cursor_base::owned> stateless(
 	trans,
-	pqxx::test::select_series(trans.conn(), 0, 9),
+	"SELECT generate_series(0, 9)",
 	"stateless",
 	false);
 
