@@ -2,7 +2,7 @@
 
 #include <pqxx/util>
 
-using namespace PGSTD;
+using namespace std;
 using namespace pqxx;
 
 namespace
@@ -11,11 +11,7 @@ void test_thread_safety_model(transaction_base &)
 {
   const thread_safety_model model = describe_thread_safety();
 
-  if (model.have_safe_strerror &&
-      model.safe_libpq &&
-      model.safe_query_cancel &&
-      model.safe_result_copy &&
-      model.safe_kerberos)
+  if (model.safe_libpq && model.safe_kerberos)
     PQXX_CHECK_EQUAL(
 	model.description,
 	"",
